<?php

use idoit\Module\Feed\ProxyApi\VirtualDevice;

/**
 * SHD SM-VIEW Feed Virtual Device API
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class feed_api_model_virtual_device extends feed_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'virtual_device';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_FEED_VIRTUAL_DEVICE';

    /**
     * Virtual device instance.
     *
     * @var VirtualDevice
     */
    private $virtual_device;

    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->virtual_device = new VirtualDevice();
    }

    /**
     * Read objects.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'filter' => array|null,
     * )
     *
     * @return array
     */
    public function read($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        $conditions = [];
        if (isset($p_params["filter"])) {
            foreach ($p_params["filter"] as $property => $value) {
                $conditions[] = "$property: *$value*";
            }
        } else {
            $conditions = ["*.*"];
        }
        $limit = ! empty($p_params["limit"]) ? $p_params["limit"] : 9999;
        $offset = ! empty($p_params["offset"]) ? $p_params["offset"] : null;
        try {
            return $this->virtual_device->query_devices($conditions, $limit, $offset);
        } catch (isys_exception_api $e) {
            return $this->return_error($this->module_name, 'read', 'could not read device - ' . $e->getMessage());
        }
    }

    /**
     * Get all fields.
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_general
     */
    public function get_fields()
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        try {
            return $this->virtual_device->get_all_fields();
        } catch (Exception $e) {
            return $this->return_error($this->module_name, 'get_fields', 'could not get fields - ' . $e->getMessage());
        }
    }

    /**
     * Create virtual device.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'user_name'   => string,
     *     'mac_address' => string,
     *     'host_name'   => string,
     *     'ip_address'  => string,
     *     'subnet'      => string,
     *     'device_name' => string,
     *     'device'      => string,
     *     'port'        => string,
     *     'port_name'   => string,
     *     'port_state'  => string,
     *     'vtp_domain'  => string,
     *     'vlan'        => string,
     *     'vlan_type'   => string,
     *     'last_seen'   => string,
     *     'notes'       => string,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_general
     */
    public function create($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::CREATE, strtolower($this->category_name));
        }

        $required_parameters = [
            'mac_address',
            'last_seen',
        ];
        $optional_parameters = [
            'user_name',
            'host_name',
            'ip_address',
            'subnet',
            'device_name',
            'device',
            'port',
            'port_name',
            'port_state',
            'vtp_domain',
            'vlan',
            'vlan_type',
            'notes',
        ];
        $data = $p_params['data'];

        try {
            $this->check_required_parameters($required_parameters, $data);
        } catch (isys_exception_validation $ex) {
            return $this->return_error($this->module_name, 'create', 'could not create object - ' . $ex->getMessage());
        }

        $params = [];
        foreach ($required_parameters as $required_parameter) {
            $params[$required_parameter] = $data[$required_parameter];
        }
        foreach ($optional_parameters as $optionalParam) {
            if (isset($data[$optionalParam])) {
                $params[$optionalParam] = $data[$optionalParam];
            }
        }

        if (empty($params['host_name'])) {
            // use ip- or mac-address if host name is empty
            $params['host_name'] = ! empty($params['ip_address']) && $params['ip_address'] != '0.0.0.0' ? $params['ip_address'] : $params['mac_address'];
        }

        $params['last_seen'] = str_replace('/', '-', $params['last_seen']);
        try {
            return $this->virtual_device->create_device($params);
        } catch (isys_exception_api $e) {
            if ($e->getCode() == 409) {
                return $this->return_error($this->module_name, 'create', 'device already exists: ' . $params['mac_address']);
            } else {
                return $this->return_error($this->module_name, 'create', 'could not create device ' . $params['mac_address'] . ': ' . $e->getMessage());
            }
        }
    }

    /**
     * Update virtual device.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'user_name'   => string,
     *     'mac_address' => string,
     *     'host_name'   => string,
     *     'ip_address'  => string,
     *     'subnet'      => string,
     *     'device_name' => string,
     *     'device'      => string,
     *     'port'        => string,
     *     'port_name'   => string,
     *     'port_state'  => string,
     *     'vtp_domain'  => string,
     *     'vlan'        => string,
     *     'vlan_type'   => string,
     *     'last_seen'   => string,
     *     'notes'       => string,
     * )
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_general
     */
    public function update($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::EDIT, strtolower($this->category_name));
        }

        $parameters = [
            'mac_address',
            'last_seen',
            'user_name',
            'host_name',
            'ip_address',
            'subnet',
            'device_name',
            'device',
            'port',
            'port_name',
            'port_state',
            'vtp_domain',
            'vlan',
            'vlan_type',
            'notes',
        ];

        $id = $p_params['mac_address'];
        $data = $p_params['data'];
        $params = [];
        foreach ($parameters as $p) {
            $params[$p] = $data[$p];
        }

        try {
            return $this->virtual_device->update_device($id, $params);
        } catch (isys_exception_api $e) {
            return $this->return_error($this->module_name, 'update', "could not update device $id: " . $e->getMessage());
        }
    }

    /**
     * Delete virtual device.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'mac_address' => string,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_general
     */
    public function delete($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::DELETE, strtolower($this->category_name));
        }

        $id = $p_params['mac_address'];
        try {
            return $this->virtual_device->delete_device($id);
        } catch (isys_exception_api $e) {
            return $this->return_error($this->module_name, 'delete', "could not delete device $id: " . $e->getMessage());
        }
    }

    /**
     * Delete all virtual device.
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_general
     */
    public function delete_all()
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::DELETE, strtolower($this->category_name));
        }

        try {
            return $this->virtual_device->re_index();
        } catch (isys_exception_api $e) {
            return $this->return_error($this->module_name, 'delete_all', "could not delete all devices: " . $e->getMessage());
        }
    }
}
