import * as React from 'react'

import { Row } from './Row'
import { RowReadonly } from './RowReadonly'
import { Select } from './Select'
import { FilterRow } from './FilterRow'
import { FilterRowReadonly } from './FilterRowReadonly'

interface AbstractProps {
    id: number,
    field_id: string,
}

export interface MappingProps extends AbstractProps {
    cat_const: string,
    property_name: string,
    property_title?: string,
}

export interface FilterProps extends AbstractProps {
    condition: string,
    value: string,
}

export interface FieldProps {
    id: string,
    name: string,
    type: string,
}

export interface CatProps {
    id: number,
    const: string,
    title: string,
}

interface MapperComposeProps {
    ajax_url: string,
    image_url: string,
    session: string,
    language: {[key: string]: string},
    fields: FieldProps[],
    cat_g_list: CatProps[],
    cat_s_list: CatProps[],
    mappings: MappingProps[],
    filters: FilterProps[],
    is_editable: boolean,
}

interface MapperComposeState {
    mappings: MappingProps[],
    filters: FilterProps[],
    has_field_selected: boolean,
}

export class MapperCompose extends React.Component<MapperComposeProps, MapperComposeState> {
  constructor (props: MapperComposeProps) {
    super(props)

    this.state = {
      mappings: this.props.mappings,
      filters: this.props.filters,
      has_field_selected: false
    }
    this.handleFieldChange = this.handleFieldChange.bind(this)
    this.handleCatChange = this.handleCatChange.bind(this)
    this.handleCatPropertyChange = this.handleCatPropertyChange.bind(this)
    this.handleMappingChange = this.handleMappingChange.bind(this)
    this.handleFieldSelectionChange = this.handleFieldSelectionChange.bind(this)
    this.addMapping = this.addMapping.bind(this)
    this.saveMappingState = this.saveMappingState.bind(this)
    this.saveFieldSelectionState = this.saveFieldSelectionState.bind(this)
    this.handleDeleteMapping = this.handleDeleteMapping.bind(this)

    this.handleFilterConditionChange = this.handleFilterConditionChange.bind(this)
    this.handleFilterValueChange = this.handleFilterValueChange.bind(this)
    this.addFilterGroup = this.addFilterGroup.bind(this)
    this.addFilter = this.addFilter.bind(this)
    this.getFilterGroups = this.getFilterGroups.bind(this)
    this.saveFilterState = this.saveFilterState.bind(this)
    this.handleDeleteFilter = this.handleDeleteFilter.bind(this)
  }

  handleFieldChange (mapping_id: number, id: number) {
    this.handleMappingChange(mapping_id, 'field_id', id.toString())
  }

  handleCatChange (mapping_id: number, id: string) {
    this.handleMappingChange(mapping_id, 'cat_const', id)
    this.handleMappingChange(mapping_id, 'property_name', '-1')
  }

  handleCatPropertyChange (mapping_id: number, id: string) {
    this.handleMappingChange(mapping_id, 'property_name', id)
  }

  handleMappingChange (mapping_id: number, property_name: string, id: string) {
    const mappings: MappingProps[] = []
    for (const key in this.state.mappings) {
      if (!this.state.mappings.hasOwnProperty(key)) {
        continue
      }

      const mapping: MappingProps = this.state.mappings[key]
      if (mapping.id == mapping_id) {
        switch (property_name) {
          case 'field_id': mapping.field_id = id; break
          case 'cat_const': mapping.cat_const = id; break
          case 'property_name': mapping.property_name = id; break
        }
      }
      mappings.push(mapping)
    }
    this.saveMappingState(mappings)
  }

  handleFieldSelectionChange () {
    const select = (document.getElementById('field-selector') as HTMLSelectElement)
    const field_id = select.options[select.selectedIndex].value
    this.saveFieldSelectionState(field_id != '-1')
  }

  addMapping () {
    // get selected field
    const select = (document.getElementById('field-selector') as HTMLSelectElement)
    const field_id = select.options[select.selectedIndex].value
    const last_id = this.get_next_id(this.state.mappings)

    // add mapping
    const mapping: MappingProps = {
      id: last_id,
      field_id: field_id,
      cat_const: '',
      property_name: null
    }
    const mappings: MappingProps[] = this.state.mappings
    mappings.push(mapping)
    this.saveMappingState(mappings)
    this.saveFieldSelectionState(false)
  }

  handleFilterConditionChange (filter_id: number, condition: string) {
    this.handleFilterChange(filter_id, 'condition', condition)
  }

  handleFilterValueChange (filter_id: number, value: string) {
    this.handleFilterChange(filter_id, 'value', value)
  }

  handleFilterChange (filter_id: number, property_name: string, id: string) {
    const filters: FilterProps[] = []
    for (const key in this.state.filters) {
      if (!this.state.filters.hasOwnProperty(key)) {
        continue
      }

      const filter: FilterProps = this.state.filters[key]
      if (filter.id == filter_id) {
        switch (property_name) {
          case 'condition': filter.condition = id; break
          case 'value': filter.value = id; break
        }
      }
      filters.push(filter)
    }
    this.saveFilterState(filters)
  }

  addFilterGroup () {
    const select = (document.getElementById('field-selector') as HTMLSelectElement)
    const field_id = select.options[select.selectedIndex].value
    this.addFilter(field_id)
  }

  addFilter (field_id: string) {
    const last_id = this.get_next_id(this.state.filters)
    const filter: FilterProps = {
      id: last_id,
      field_id: field_id,
      condition: null,
      value: ''
    }
    const filters: FilterProps[] = this.state.filters
    filters.push(filter)
    this.saveFilterState(filters)
    this.saveFieldSelectionState(false)
  }

  get_next_id (elements: AbstractProps[]) {
    let last_id = 0
    for (const key in elements) {
      if (elements.hasOwnProperty(key)) {
        const filter = elements[key]
        last_id = last_id < filter.id ? filter.id : last_id
      }
    }
    last_id++
    return last_id
  }

  getFilterGroups () {
    const filter_groups:{[key: string]: FilterProps[]} = {}
    for (const key in this.state.filters) {
      if (!this.state.filters.hasOwnProperty(key)) {
        continue
      }

      const filter = this.state.filters[key]
      if (!filter_groups[filter.field_id]) {
        filter_groups[filter.field_id] = []
      }
      filter_groups[filter.field_id].push(filter)
    }
    return filter_groups
  }

  getFilterConditions () {
    return {
      equal: this.props.language.LC__CMDB__CATG__SMVIEW_FEED__FILTER_EQUAL,
      contains: this.props.language.LC__CMDB__CATG__SMVIEW_FEED__FILTER_CONTAINS,
      not_equal: this.props.language.LC__CMDB__CATG__SMVIEW_FEED__FILTER_NOT_EQUAL,
      not_contains: this.props.language.LC__CMDB__CATG__SMVIEW_FEED__FILTER_NOT_CONTAINS,
    }
  }

  handleDeleteMapping (mapping_id: number) {
    const mappings: MappingProps[] = []
    for (const key in this.state.mappings) {
      if (!this.state.mappings.hasOwnProperty(key)) {
        continue
      }

      const mapping = this.state.mappings[key]
      if (mapping.id != mapping_id) {
        mappings.push(mapping)
      }
    }
    this.saveMappingState(mappings)
  }

  handleDeleteFilter (filter_id?: number, field_id?: string) {
    const filters: FilterProps[] = []
    for (const key in this.state.filters) {
      if (!this.state.filters.hasOwnProperty(key)) {
        continue
      }

      const filter = this.state.filters[key]
      if (filter_id && filter.id != filter_id) {
        filters.push(filter)
      }
      if (field_id && filter.field_id != field_id) {
        filters.push(filter)
      }
    }
    this.saveFilterState(filters)
  }

  saveMappingState (mappings: MappingProps[]) {
    this.setState((prevState) => ({
      ...prevState,
      mappings: mappings
    }))
  }

  saveFilterState (filters: FilterProps[]) {
    this.setState((prevState) => ({
      ...prevState,
      filters: filters
    }))
  }

  saveFieldSelectionState (is_selected: boolean) {
    this.setState((prevState) => ({
      ...prevState,
      has_field_selected: is_selected
    }))
  }

  static sortByValue (sortables: string[][]) {
    sortables.sort(function (a, b) {
      return (a[1].toString() > b[1].toString()) ? 1 : -1
    })

    const categories:{[key: string]: string} = {}
    for (const c in sortables) {
      if (sortables.hasOwnProperty(c)) {
        const cat = sortables[c]
        categories[cat[0]] = cat[1]
      }
    }
    return categories
  }

  render () {
    const sortable_fields = []
    for (const key in this.props.fields) {
      if (this.props.fields.hasOwnProperty(key)) {
        const field = this.props.fields[key]
        sortable_fields.push([field.id, field.name])
      }
    }
    const fields = MapperCompose.sortByValue(sortable_fields)

    const divs = []
    for (const key in this.state.mappings) {
      if (!this.state.mappings.hasOwnProperty(key)) {
        continue
      }

      const mapping = this.state.mappings[key]
      if (this.props.is_editable) {
        divs.push(<Row key={'mapping_' + mapping.field_id}
          mapping={mapping}
          fields={this.props.fields}
          cat_g_list={this.props.cat_g_list}
          cat_s_list={this.props.cat_s_list}
          onFieldChange={this.handleFieldChange}
          onCatChange={this.handleCatChange}
          onCatPropertyChange={this.handleCatPropertyChange}
          onDeleteMapping={this.handleDeleteMapping}
          ajax_url={this.props.ajax_url}
          image_url={this.props.image_url}
          language={this.props.language}
          session={this.props.session}/>)
      } else {
        divs.push(<RowReadonly
          key={'mapping_' + mapping.field_id}
          mapping={mapping}
          fields={this.props.fields}
          cat_g_list={this.props.cat_g_list}
          cat_s_list={this.props.cat_s_list}
          language={this.props.language} />)
      }

      // remove used fields
      if (fields[mapping.field_id]) {
        delete fields[mapping.field_id]
      }
    }

    const filter_groups = this.getFilterGroups()
    for (const key in filter_groups) {
      if (!filter_groups.hasOwnProperty(key)) {
        continue
      }

      const filters = filter_groups[key]
      if (this.props.is_editable) {
        divs.push(<FilterRow
          key={'filter_' + key}
          field_id={key}
          filters={filters}
          fields={this.props.fields}
          conditions={this.getFilterConditions()}
          onAdd={this.addFilter}
          onConditionChange={this.handleFilterConditionChange}
          onValueChange={this.handleFilterValueChange}
          onDelete={this.handleDeleteFilter}
          image_url={this.props.image_url}
          language={this.props.language} />)
      } else {
        divs.push(<FilterRowReadonly
          key={'filter_' + key}
          field_id={key}
          filters={filters}
          fields={this.props.fields}
          conditions={this.getFilterConditions()}
          language={this.props.language} />)
      }

      // remove used fields
      if (fields[key]) {
        delete fields[key]
      }
    }

    let selector
    if (this.props.is_editable) {
      const add_img = this.props.image_url + 'add.png'
      selector = <div>
        <div className="border mr10 mb5 field-selection">
          <h3 className="gradient p5 mouse-pointer">
            <span>{this.props.language.LC__CMDB__CATG__SMVIEW_FEED__ADD_ASSIGNMENT}</span>
          </h3>
          <div className="p5 border-top">
            <div className="property-label p5">
              <strong className="text-shadow">{this.props.language.LC__CMDB__CATG__SMVIEW_FEED__FIELD_NAME}</strong>
            </div>
            <Select id="field-selector" name="field-selector" options={fields} onChange={this.handleFieldSelectionChange} />
            <button type="button" className="mt5 btn btn-block fl m5"
              onClick={this.addMapping} disabled={!this.state.has_field_selected}>
              <img src={add_img} alt="+" className="mr5" />
              <span>{this.props.language.LC__CMDB__CATG__SMVIEW_FEED__ADD_FIELD_ASSIGNMENT}</span>
            </button>
            <button type="button" className="mt5 btn btn-block fl m5"
              onClick={this.addFilterGroup} disabled={!this.state.has_field_selected}>
              <img src={add_img} alt="+" className="mr5" />
              <span>{this.props.language.LC__CMDB__CATG__SMVIEW_FEED__ADD_FILTER_ASSIGNMENT}</span>
            </button>
          </div>
        </div>
      </div>
    }

    return <div className="m10">
      {selector}
      {divs}
    </div>
  }
}
