import * as React from 'react'

import { Select } from './Select'
import { CatProps, FieldProps, MappingProps, MapperCompose } from './MapperCompose'

interface RowProps {
    ajax_url: string,
    image_url: string,
    session: string,
    language: {[key: string]: string},
    fields: FieldProps[],
    cat_g_list: CatProps[],
    cat_s_list: CatProps[],
    mapping: MappingProps,
    onFieldChange: Function,
    onCatChange: Function,
    onCatPropertyChange: Function,
    onDeleteMapping: Function,
}

interface IHash {
    [key: string]: string
}

interface RowState {
    cat_properties: IHash
}

export class Row extends React.Component<RowProps, RowState> {
  constructor (props: RowProps) {
    super(props)

    this.state = {
      cat_properties: {}
    }
    this.handleFieldChange = this.handleFieldChange.bind(this)
    this.handleCatChange = this.handleCatChange.bind(this)
    this.handleCatPropertyChange = this.handleCatPropertyChange.bind(this)
    this.loadCatProperties = this.loadCatProperties.bind(this)
    this.handleDeleteMapping = this.handleDeleteMapping.bind(this)
  }

  componentDidMount () {
    this.loadCatProperties(this.props.mapping.cat_const)
  }

  handleFieldChange (id: string) {
    this.props.onFieldChange(this.props.mapping.id, id)
  }

  handleCatChange (cat_const: string) {
    this.props.onCatChange(this.props.mapping.id, cat_const)
    this.loadCatProperties(cat_const)
  }

  handleCatPropertyChange (event: React.ChangeEvent<HTMLInputElement>) {
    const prop_id: string = event.currentTarget.value
    this.props.onCatPropertyChange(this.props.mapping.id, prop_id)
  }

  handleDeleteMapping () {
    this.props.onDeleteMapping(this.props.mapping.id)
  }

  loadCatProperties (cat_const: string) {
    const url = this.props.ajax_url
    loadAjax(url, this.props.session, cat_const)
      .then(res => {
        const values = { ...res }
        const cat_properties: IHash = {}
        for (const key in values) {
          if (!values.hasOwnProperty(key)) {
            continue
          }

          const ident: Array<string> = key.split('#')
          if (ident[0].startsWith('custom_')) {
            // do not add custom properties
            continue
          }
          cat_properties[ident[0]] = values[key]
        }

        this.setState((prevState) => ({
          ...prevState,
          cat_properties: cat_properties
        }))
      })
      .catch(() => {
        this.setState((prevState) => ({
          ...prevState,
          cat_properties: {}
        }))
      })
  }

  render () {
    let title = ''
    for (const key in this.props.fields) {
      if (!this.props.fields.hasOwnProperty(key)) {
        continue
      }
      const field = this.props.fields[key]
      if (field.id == this.props.mapping.field_id) {
        title = field.name
      }
    }

    // add all categories
    const sortable_categories = []
    for (const key in this.props.cat_g_list) {
      if (this.props.cat_g_list.hasOwnProperty(key)) {
        const cat = this.props.cat_g_list[key]
        sortable_categories.push([cat.const, cat.title])
      }
    }
    for (const key in this.props.cat_s_list) {
      if (this.props.cat_s_list.hasOwnProperty(key)) {
        const cat = this.props.cat_s_list[key]
        sortable_categories.push([cat.const, cat.title])
      }
    }
    const categories = MapperCompose.sortByValue(sortable_categories)

    // add all category properties
    const sortable_properties = []
    for (const key in this.state.cat_properties) {
      if (this.state.cat_properties.hasOwnProperty(key)) {
        const cat = this.state.cat_properties[key]
        sortable_properties.push([key, cat])
      }
    }
    sortable_properties.sort(function (a, b) {
      return (a[1].toString() > b[1].toString()) ? 1 : -1
    })

    const lis = []
    for (const key in sortable_properties) {
      if (!sortable_properties.hasOwnProperty(key)) {
        continue
      }

      const prop = sortable_properties[key]
      const name = 'cat_property_' + this.props.mapping.id
      lis.push(<li key={name + '_' + prop[0]}>
        <input type="radio" name={name} value={prop[0]} checked={prop[0] == this.props.mapping.property_name} onChange={this.handleCatPropertyChange} />
        <span>{prop[1]}</span>
      </li>)
    }

    const field_name = 'target_field_' + this.props.mapping.id
    const cat_name = 'cat_selection_' + this.props.mapping.id
    const delete_img = this.props.image_url + 'cross.png'
    return <div className="border mt10 category-container">
      <h4 className="gradient p5 mouse-pointer">
        <button type="button" className="btn btn-small close fr" onClick={this.handleDeleteMapping} title={this.props.language.LC__CMDB__CATG__SMVIEW_FEED__DELETE_ASSIGNMENT}>
          <img src={delete_img} alt="x" />
        </button>
        <span>{title}</span>
        <input type="hidden" name={field_name} value={this.props.mapping.field_id} />
      </h4>

      <div className="border-top">
        <div className="property-label">
          <strong className="text-shadow">{this.props.language.LC__CMDB__CATG__SMVIEW_FEED__CATEGORY}</strong>
        </div>
        <div className="fl">
          <Select name={cat_name} options={categories} selected_option={this.props.mapping.cat_const} onChange={this.handleCatChange} />
        </div>
      </div>
      <div>
        <div className="property-label">
          <strong className="text-shadow">{this.props.language.LC__CMDB__CATG__SMVIEW_FEED__PROPERTY}</strong>
        </div>
        <div className="fl">
          <ul>
            {lis}
          </ul>
        </div>
      </div>
    </div>
  }
}

async function loadAjax (ajax_url: string, session: string, cat_const: string) {
  const formData = new FormData()
  formData.append('cat_const', cat_const)
  formData.append('cat_type', 'g')

  const res = await fetch(ajax_url, {
    method: 'POST',
    headers: {
      'X-RPC-Auth-Session': session
    },
    credentials: 'same-origin',
    body: formData
  })
  return await res.json()
}
