<?php

use idoit\Module\Feed\DaoListResult;

/**
 * i-doit
 *
 * DAO: abstract global category for SM-VIEW Feed.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
abstract class isys_cmdb_dao_category_g_smview_feed_abstract extends isys_cmdb_dao_category_global
{

    /**
     * Field name separator to replace '.' notation.
     */
    const FIELD_NAME_SEPARATOR = '___';

    /**
     * Constant to replace empty values with.
     */
    const NO_VALUE_MATCH = '__XX__NO_VALUE_MATCHING__XX__';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * List of cached issues.
     *
     * @var array
     */
    protected $_cached_elements;

    /**
     * @var string
     */
    protected $m_object_id_field = 'isys_key';

    /**
     * Main table where properties are stored persistently.
     *
     * @var  string
     */
    protected $m_table = 'isys';

    /**
     * Prefix of field names.
     *
     * @var string
     */
    protected $_field_name_prefix = null;

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws Exception
     */
    protected function properties()
    {
        $fields = $this->_getFields();

        $properties = [];
        foreach ($fields as $f) {
            $key = $this->_field_name_prefix . strtoupper($f['name']);
            $field_type = $f['type'] == 'number' ? isys_cmdb_dao_category_pattern::int() : isys_cmdb_dao_category_pattern::text();
            $field_id = 'isys_' . $f['id'];
            $properties[$field_id] = array_replace_recursive(
                $field_type,
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => $f['name'],
                        C__PROPERTY__INFO__DESCRIPTION => $f['name']
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => $field_id
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => $key
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH => true,
                        C__PROPERTY__PROVIDES__LIST   => true,
                    ]
                ]
            );
        }
        return $properties;
    }

    /**
     * Get all fields of this object.
     *
     * @return array
     */
    abstract protected function _getFields();

    /**
     * Retrieves the number of saved category-entries to the given object.
     *
     * @param  integer $objectId
     * @return integer
     * @throws Exception
     */
    public function get_count($objectId = null)
    {
        return count($this->_getElements($objectId));
    }

    /**
     * Retrieves general data. Query runs every time you run this method. And it will always return the first row from the result set. Returns null on error.
     *
     * @throws  Exception
     * @return  array
     */
    public function get_general_data()
    {
        if (! $this->m_daores || $this->m_daores->count() > 1) {
            return null;
        }
        return $this->m_daores->get_row();
    }

    /**
     * Fetches category data from database.
     *
     * @param  integer $p_category_data_id
     * @param  mixed $p_obj_id May be an integer, or an array of integers.
     * @param  string $p_condition
     * @param  mixed $p_filter
     * @param  integer $p_status
     *
     * @return isys_component_dao_result
     * @throws Exception
     */
    public function get_data($p_category_data_id = null, $p_obj_id = null, $p_condition = '', $p_filter = null, $p_status = null)
    {
        $this->m_daores = new DaoListResult($this->_getElements($p_obj_id), $p_category_data_id);
        return $this->m_daores;
    }

    /**
     * Get elements, filtered by object ID.
     *
     * @param  int|null $p_obj_id
     * @return array
     * @throws Exception
     */
    abstract protected function _getElements($p_obj_id = null);

    /**
     * Get query conditions.
     *
     * @param  int $p_obj_id
     * @param  isys_cmdb_dao_smview_feed_abstract $mapping_dao
     * @param  isys_cmdb_dao_smview_feed_abstract $filter_dao
     * @return array
     * @throws \idoit\Exception\Exception
     * @throws isys_exception_database
     */
    protected function _get_conditions($p_obj_id, $mapping_dao, $filter_dao)
    {
        $conditions = [];
        if ($mapping_dao) {
            $fields = $mapping_dao->get_fields_by_obj_id($p_obj_id);
            foreach ($fields as $field) {
                $values = $this->_get_category_values($p_obj_id, $field);
                $field_name = $this->_get_field_id($field['field_id']);
                if (count($values) > 1) {
                    $condition_group = [];
                    foreach ($values as $value) {
                        if ($value != '') {
                            $condition_group[] = "$field_name: \"$value\"";
                        }
                    }
                    $group = implode(' OR ', $condition_group);
                    $conditions[] = "($group)";
                } elseif (count($values) == 1) {
                    $value = $values[0];
                    $value = $value == '' ? self::NO_VALUE_MATCH : $value;
                    $conditions[] = "$field_name: \"$value\"";
                }
            }
        }

        if ($filter_dao) {
            $fields = $filter_dao->get_fields_by_obj_id($p_obj_id);
            foreach ($fields as $field) {
                $field_name = $this->_get_field_id($field['field_id']);
                $value = $field['value'];
                switch ($field['condition']) {
                    case 'equal': $conditions[] = "$field_name: \"$value\""; break;
                    case 'contains': $conditions[] = "$field_name: *$value*"; break;
                    case 'not_equal': $conditions[] = "NOT $field_name: \"$value\""; break;
                    case 'not_contains': $conditions[] = "NOT $field_name: *$value*"; break;
                }
            }
        }
        return $conditions;
    }

    /**
     * Get category value.
     *
     * @param  int $p_obj_id
     * @param  array $field
     * @return array
     * @throws \idoit\Exception\Exception
     * @throws isys_exception_database
     */
    private function _get_category_values($p_obj_id, $field)
    {
        $catg_id = $field['catg_id'];
        $cats_id = $field['cats_id'];
        $property_name = $field['property_name'];

        $lang = isys_application::instance()->container->get('language');
        $dao = isys_cmdb_dao_category_g_global::instance($this->m_db);
        $sql = '';
        if (! empty($cats_id)) {
            $sql = "SELECT
                s.isysgui_cats__const as cat_const,
                s.isysgui_cats__class_name as class_name,
                s.isysgui_cats__source_table as source_table
                FROM isysgui_cats s WHERE s.isysgui_cats__id = " . $cats_id;
        }
        if (! empty($catg_id)) {
            $sql = "SELECT
                g.isysgui_catg__const as cat_const,
                g.isysgui_catg__class_name as class_name,
                g.isysgui_catg__source_table as source_table
                FROM isysgui_catg g WHERE g.isysgui_catg__id = " . $catg_id;
        }
        $row = $dao->retrieve($sql)->get_row();
        if ($row['cat_const'] == 'C__CATG__GLOBAL') {
            return array($this->_get_value_of_object($p_obj_id, $property_name));
        }

        $class_name = $row['class_name'];
        if (! class_exists($class_name)) {
            $message = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__ERROR_UNKNOWN_CLASS') . ": $class_name";
            throw new \idoit\Exception\Exception($message);
        }

        $cat_dao = new $class_name($this->m_db);
        $cat_data = $cat_dao->get_data_as_array(null, $p_obj_id);
        if (empty($cat_data)) {
            $message = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__ERROR_EMPTY_CATEGORY') . ": $class_name";
            throw new \idoit\Exception\Exception($message);
        }

        // get properties of category
//        $l_dao = new isys_cmdb_dao_category_property(isys_application::instance()->database);
//        $l_res = $l_dao->retrieve_properties(
//            null,
//            null,
//            null,
//            null,
//            'AND isys_property_2_cat__cat_const = ' . $l_dao->convert_sql_text($row['cat_const']),
//            null);
//        $rows = [];
//        while ($row = $l_res->get_row()) {
//            $rows[$row['key']] = $row;
//        }

        $cat_props = $cat_dao->get_properties();
        $values = [];
        $foundProp = false;
        foreach ($cat_data as $data) {
            $source_table = $row['source_table'];
            $prop_name = $source_table . '_list__' . $property_name;
            if (! isset($data[$prop_name])) {
                $prop_name = $source_table . '__' . $property_name;
            }

            if (! isset($data[$prop_name])) {
                // get field by property name
                $prop = $cat_props[$property_name];
                $prop_data = $prop[C__PROPERTY__DATA];
                $prop_name = $prop_data[C__PROPERTY__DATA__FIELD];
            }

            if (! isset($data[$prop_name])) {
                // try to resolve reference
                $prop = $cat_props[$property_name];
                $prop_data = $prop[C__PROPERTY__DATA];
                if ($prop_data[C__PROPERTY__DATA__REFERENCES]) {
                    $refs = $prop_data[C__PROPERTY__DATA__REFERENCES];
                    $ref = count($refs) > 1 ? $refs[1] : '';
                    $prop_name = str_replace('__id', '__title', $ref);
                    if (isset($data[$prop_name])) {
                        $values[] = $data[$prop_name];
                        $foundProp = true;
                    }
                }
            } else {
                $values[] = $data[$prop_name];
                $foundProp = true;
            }
        }

        if (! $foundProp) {
            $message = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__ERROR_MISSING_PROPERTY') . ": $property_name";
            throw new \idoit\Exception\Exception($message);
        }
        return $values;
    }

    /**
     * Get value from object table.
     *
     * @param  int $p_obj_id
     * @param  string $property_name
     * @return string
     * @throws \idoit\Exception\Exception
     * @throws Exception
     */
    private function _get_value_of_object($p_obj_id, $property_name)
    {
        $dao = isys_cmdb_dao_category_g_global::instance($this->m_db);
        $row = $dao->get_data(null, $p_obj_id)->get_row();
        $key = 'isys_obj__' . $property_name;
        if (! isset($row[$key])) {
            $lang = isys_application::instance()->container->get('language');
            $message = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__ERROR_MISSING_PROPERTY') . ": $key";
            throw new \idoit\Exception\Exception($message);
        }
        return $row[$key];
    }

    /**
     * Map field ID from internal separator back to '.' notation.
     *
     * @param  string $field_name
     * @return string
     */
    abstract protected function _get_field_id($field_name);

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     */
    public function create_data($p_data)
    {
        return false;
    }

    /**
     * Updates existing entity.
     *
     * @param  integer $p_bsm_list_id Entity's identifier
     * @param  array   $p_data        Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     */
    public function save_data($p_bsm_list_id, $p_data)
    {
        return false;
    }
}
