<?php

use idoit\Module\Feed\ProxyApi\VirtualDevice;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Feed Virtual Device.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2020 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_feed_virtual_device extends isys_cmdb_dao_category_g_smview_feed_abstract
{

    /**
     * Ignored virtual device properties.
     */
    const IGNORED_FIELDS = ['user_name', 'vtp_domain', 'vlan', 'vlan_type', 'notes'];

    /**
     * Regular expressions to replace virtual device port names.
     */
    const PORT_NAME_REGEX = [
        '/Gi\d\/\d{1,3}\/(\d{1,3})/' => '$1',
        '/Gi\d\/(\d{1,3})/'          => '$1',
//        '/Fa\d\/(\d{1,3})/'          => '$1',
//        '/\d\/(\d{1,3})/'            => '$1',
    ];

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_feed_virtual_device';

    /**
     * Prefix of field names.
     *
     * @var string
     */
    protected $_field_name_prefix = 'C__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_';

    /**
     * Retrieves the number of saved category-entries to the given object.
     *
     * @param  int $objectId
     * @return int
     * @throws Exception
     */
    public function get_count($objectId = null)
    {
        $virtualDeviceCount = count($this->_getElements($objectId));

        $dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $filter = ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . $dao->convert_sql_int(C__CONNECTOR__OUTPUT);
        $connectorCount = $dao->get_data(null, $objectId, $filter)->count();

        // return 0 if this object has no outputs
        return $connectorCount > 0 ? $virtualDeviceCount : 0;
    }


    /**
     * Get all fields of this object.
     *
     * @return array
     * @throws Exception
     */
    protected function _getFields()
    {
        $language = isys_application::instance()->container->get('language');
        try {
            $proxy_api = new VirtualDevice();
        } catch (isys_exception_api $e) {
            isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_BACKEND_UNREACHABLE'));
            return [];
        }
        $fields = [];
        foreach ($proxy_api->get_all_fields() as $f) {
            if (! in_array($f['id'], self::IGNORED_FIELDS)) {
                $fields[] = $f;
            }
        }
        return $fields; //$proxy_api->get_all_fields();
    }

    /**
     * Get issues from OrientDB, filtered by object ID.
     *
     * @param  int|null $p_obj_id
     * @return array
     * @throws Exception
     */
    protected function _getElements($p_obj_id = null)
    {
        if ($this->_cached_elements === null || ! isset($this->__cached_elements[$p_obj_id])) {
            $language = isys_application::instance()->container->get('language');
            $mapping_dao = new isys_cmdb_dao_smview_feed_virtual_device_2_obj_type($this->m_db);
            try {
                $conditions = ! empty($p_obj_id) ? $this->_get_conditions($p_obj_id, $mapping_dao, null) : [];
            } catch (Exception $e) {
                isys_application::instance()->container['notify']->notice($e->getMessage());
                $this->_cached_elements[$p_obj_id] = [];
                return [];
            }

            if (empty($conditions)) {
                $this->_cached_elements[$p_obj_id] = [];
                return [];
            }

            try {
                $proxy_api = new VirtualDevice();
            } catch (isys_exception_api $e) {
                isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_BACKEND_UNREACHABLE'));
                return [];
            } catch (isys_exception_database $e) {
                isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_CONFIGURATION_INVALID'));
                return [];
            }

            try {
                $this->_cached_elements[$p_obj_id] = $proxy_api->query_devices($conditions);
            } catch (isys_exception_api $e) {
                isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_FEED__CACHE_BACKEND_UNREACHABLE'));
                return [];
            }
        }
        return $this->_cached_elements[$p_obj_id];
    }

    /**
     * Map field ID from internal separator back to '.' notation.
     *
     * @param  string $field_name
     * @return string
     */
    protected function _get_field_id($field_name)
    {
        return str_replace(self::FIELD_NAME_SEPARATOR, '.', $field_name);
    }

    /**
     * Get matching connector by port name of virtual device.
     *
     * @param  string $portName
     * @return array|null
     */
    public function get_connector_by_virtual_device_port_name($portName)
    {
        $dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $objId = $this->get_object_id();
        $data = $dao->get_data_as_array(null, $objId);

        $port = $portName;
        foreach (self::PORT_NAME_REGEX as $regex => $replacement) {
            if (preg_match($regex, $portName)) {
                $port = preg_replace($regex, $replacement, $portName);
                break;
            }
        }

        foreach ($data as $p) {
            if ($p['isys_catg_connector_list__type'] == C__CONNECTOR__OUTPUT && $p['isys_catg_connector_list__title'] == $port) {
                return $p;
            }
        }
        return null;
    }

    /**
     * Get matching virtual devices by port name of connector.
     *
     * @param  string $portName
     * @param  array|null $data
     * @return array
     */
    public function get_virtual_device_ports_by_connector($portName, $data = null)
    {
        if (! $data) {
            // get virtual device data
            $data = $this->get_data_as_array(null, $this->get_object_id());
        }

        $ports = [];
        foreach ($data as $p) {
            $port = $p['isys_port'];
            // replace virtual device port name by regular expressions
            foreach (self::PORT_NAME_REGEX as $regex => $replacement) {
                if (preg_match($regex, $port)) {
                    $port = preg_replace($regex, $replacement, $port);
                }
            }

            if ($portName == $port) {
                // found matching port name
                $ports[] = $p;
            }
        }
        return $ports;
    }

    /**
     * Trace a port until last device is reached.
     *
     * @param  array $port
     * @return array
     * @throws isys_exception_database
     */
    public function tracePort($port)
    {
        // start tracing only from outputs
        $ignoreSibling = $port['isys_catg_connector_list__type'] == C__CONNECTOR__OUTPUT;
        $port = $this->_trace($port, [], $ignoreSibling);
        return [
            'id'              => $port['isys_obj__id'],
            'title'           => $port['isys_obj__title'],
            'connector_id'    => $port['isys_catg_connector_list__id'],
            'connector_title' => $port['isys_catg_connector_list__title'],
            'location'        => $this->_get_location_path($port['isys_obj__id']),
        ];
    }

    /**
     * Trace a port until last device is reached.
     *
     * @param  array $port
     * @param  array $tracedPortIds
     * @param  boolean $ignoreSibling
     * @return array
     */
    private function _trace($port, $tracedPortIds = [], $ignoreSibling = false)
    {
        $dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $tracedPortIds[] = $port['isys_catg_connector_list__id'];
        $connector = $port;
        if (! $ignoreSibling && ! empty($port['isys_catg_connector_list__isys_catg_connector_list__id'])) {
            // connector has a sibling
            $sibling_connector = $port['isys_catg_connector_list__isys_catg_connector_list__id'];
            $connector = $dao->get_data($sibling_connector)->get_row();
            $tracedPortIds[] = $connector['isys_catg_connector_list__id'];
        }

        $connected_connector = $connector['con_connector'];
        $device = $port;
        if (! empty($connected_connector) && ! in_array($connected_connector, $tracedPortIds)) {
            // trace connected connector
            $connected_port = $dao->get_data($connected_connector)->get_row();
            $device = $this->_trace($connected_port, $tracedPortIds);
        }
        return $device;
    }

    /**
     * Get location path as string.
     *
     * @param  int $obj_id
     * @return string
     * @throws isys_exception_database
     */
    private function _get_location_path($obj_id)
    {
        $location_dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);
        $location_path_ids = $location_dao->get_location_path($obj_id);
        $path = [];
        $strLen = 0;
        foreach ($location_path_ids as $i) {
            $loc = $location_dao->get_object_by_id($i)->get_row();
            $path[] = $loc['isys_obj__title'];
            $strLen += strlen($loc['isys_obj__title']);
            if ($strLen > 50) {
                break;
            }
        }

        $path = array_reverse($path);
        return implode(' > ', $path);
    }
}
