<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Feed Virtual Device.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2020 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_feed_virtual_device extends isys_cmdb_ui_category_global
{

    /**
     * Object types, having multiple ports.
     */
    const MULTI_PORT_OBJECT_TYPES = ['C__OBJTYPE__ROUTER', 'C__OBJTYPE__SWITCH'];

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_feed::get_tpl_dir() . 'module/cmdb/catg__smview_feed_virtual_device.tpl');
    }

    /**
     * Processes category data list for multi-valued categories.
     *
     * @param isys_cmdb_dao_category $p_cat
     * @param array $p_get_param_override
     * @param string $p_strVarName
     * @param string $p_strTemplateName
     * @param bool $p_bCheckbox
     * @param bool $p_bOrderLink
     * @param string $p_db_field_name
     *
     * @return null
     * @throws isys_exception_cmdb
     * @throws isys_exception_general
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    public function process_list(isys_cmdb_dao_category &$p_cat, $p_get_param_override = null, $p_strVarName = null, $p_strTemplateName = null, $p_bCheckbox = true, $p_bOrderLink = true, $p_db_field_name = null)
    {
        // get object type and switch to listing or single page
        $dao = new isys_cmdb_dao_category_g_global($this->m_database_component);
        $obj = $dao->get_data(null, $p_cat->get_object_id())->get_row();
        $count = $p_cat->get_data(null, $p_cat->get_object_id())->count();
        if (! in_array($obj['isys_obj_type__const'], self::MULTI_PORT_OBJECT_TYPES) && $count <= 1) {
            // process only single page
            return $this->process($p_cat);
        }

        // important: change category DAO to isys_cmdb_dao_category_g_connector to support listing of all local ports
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_database_component);
        // set this object ID
        $connector_dao->set_object_id($p_cat->get_object_id());
        // change listing class to support custom code
        $connector_dao->set_list('isys_cmdb_dao_list_catg_smview_feed_virtual_device');

        // continue processing
        parent::process_list($connector_dao, $p_get_param_override, $p_strVarName, $p_strTemplateName, $p_bCheckbox, $p_bOrderLink, $p_db_field_name);
        isys_component_template_navbar::getInstance()
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__NEW'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__ARCHIVE'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__QUICK_PURGE'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__PRINT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EXPORT_AS_CSV'));
        return null;
    }

    /**
     * Processes the UI for this category
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        $dao = new isys_cmdb_dao_category_g_smview_feed_virtual_device($this->m_database_component);
        parent::process($p_cat);
        $this->disable_ajax_save();
        $this->deactivate_commentary();

        // important: $this->m_category_data_id matches isys_catg_connector_list__id because of
        // changed DAO from process_list
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_database_component);
        $catGId = (int) $this->m_category_data_id;
        $catGId = $catGId > 1 ? $catGId : null;
        if ($catGId) {
            $connector = $connector_dao->get_data($catGId)->get_row();
            $endDevice = $dao->tracePort($connector);
            $endDevice['url'] = isys_helper_link::get_base() . isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT => $endDevice['id'],
                    C__CMDB__GET__CATG   => defined_or_default('C__CATG__CONNECTOR'),
                ]);
            $endDevice['connector_url'] = isys_helper_link::get_base() . isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT => $endDevice['id'],
                    C__CMDB__GET__CATG   => defined_or_default('C__CATG__CONNECTOR'),
                    C__CMDB__GET__CATLEVEL   => $endDevice['connector_id'],
                ]);

            $this->get_template_component()
                ->assign('port', $connector)
                ->assign('end_device', $endDevice);
        }

        // try to get matching virtual device by local connector title
        $obj_id = $p_cat->get_object_id();
        $virtual_device_data = $dao->get_data_as_array(null, $obj_id);
        if (isset($connector)) {
            // filter matching ports
            $devices = $dao->get_virtual_device_ports_by_connector($connector['isys_catg_connector_list__title'], $virtual_device_data);
        } else {
            // use all virtual devices
            $devices = $virtual_device_data;
        }
        if ($devices) {
            $properties = $p_cat->get_properties();
            $this->get_template_component()
                ->assign('properties', $properties)
                ->assign('devices', $devices);
        } else {
            $this->get_template_component()->assign('properties', []);
        }

        isys_component_template_navbar::getInstance()
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        return $language->get('LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_TITLE');
    }
}
