<?php

namespace idoit\Module\Feed\Connectors;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use isys_application;
use isys_cmdb_dao_smview_feed_jira;
use isys_exception_database;

/**
 * Jira connector class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Jira
{

    /**
     * JIRA server configuration.
     *
     * @var array
     */
    private $_jira_server = [];

    /**
     * Constructor.
     *
     * @param  int|null $config_instance_id
     * @throws isys_exception_database
     */
    public function __construct($config_instance_id = null)
    {
        $dao = new isys_cmdb_dao_smview_feed_jira(isys_application::instance()->database);
        if ($config_instance_id) {
            $this->_jira_server = $dao->get_object('id', $config_instance_id);
        } else {
            $configs = $dao->get_objects();
            $this->_jira_server = $configs[0];
        }
    }

    /**
     * Get all issues from JIRA instance.
     *
     * @return array
     * @throws GuzzleException
     */
    public function get_issues_from_instance()
    {
        $client = new Client();
        $uri = $this->_jira_server['address'] . '/rest/api/2/search?jql&maxResults=2000'; //AND issuetype="Bug" AND resolution="Unresolved"';
        $response = $client->request('GET', $uri, [
            'auth'   => [$this->_jira_server['username'], $this->_jira_server['password']],
            'verify' => false,
        ]);

        if ($response->getStatusCode() != 200 || ! $response->getBody()) {
            // error
            return [];
        }

        $json = '';
        while ($part = $response->getBody()->read(512)) {
            $json .= $part;
        }

        $obj = json_decode($json, true);
        $issues = [];
        foreach ($obj['issues'] as $issue) {
            $issues[$issue['id']] = $this->_serialize_issue($issue);
        }
        return $issues;
    }

    /**
     * Serialize issue.
     *
     * @param  array $issue
     * @return array
     */
    private function _serialize_issue($issue)
    {
        $data = [
            'id'   => $issue['id'],
            'key'  => $issue['key'],
            'self' => $issue['self'],
        ];

        foreach ($issue['fields'] as $id => $field) {
            $value = $field;
            if (is_array($field)) {
                $value = $field['name'] ?: null;
            }
            $data[$id] = $value;
        }
        return $data;
    }

    /**
     * Get all fields of an issue.
     *
     * @return array
     * @throws GuzzleException
     */
    public function get_fields()
    {
        $client = new Client();
        $uri = $this->_jira_server['address'] . '/rest/api/2/field';
        $response = $client->request('GET', $uri, [
            'auth'   => [$this->_jira_server['username'], $this->_jira_server['password']],
            'verify' => false,
        ]);

        if ($response->getStatusCode() != 200 || ! $response->getBody()) {
            // error
            return [];
        }

        $json = '';
        while ($part = $response->getBody()->read(512)) {
            $json .= $part;
        }

        $field_data = json_decode($json, true);
        $fields = [];
        foreach ($field_data as $field) {
            if (stripos($field['id'], 'customfield_') === 0) {
                // ignore custom fields
                continue;
            }
            $fields[$field['id']] = $this->_serialize_field($field);
        }
        return $fields;
    }

    /**
     * Serialize a field.
     *
     * @param  array $field
     * @return array
     */
    private function _serialize_field($field)
    {
        $type = isset($field['schema']) ? $field['schema']['type'] : null;
        return [
            'id'     => $field['id'],
            'name'   => $field['name'],
            'custom' => $field['custom'] == 'true',
            'type'   => $type,
        ];
    }
}
