<?php

namespace idoit\Module\Feed\Connectors;

use isys_application;
use isys_cmdb_dao_smview_feed_configuration;
use PhpOrient\PhpOrient;

/**
 * OrientDB class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class OrientDB
{
    /**
     * OrientDB client.
     *
     * @var PhpOrient
     */
    private $_client;

    /**
     * OrientDB constructor.
     */
    public function __construct()
    {
        $dao = new isys_cmdb_dao_smview_feed_configuration(isys_application::instance()->database);
        $config = $dao->get_object('id', 1);

        if (empty($config['address'])) {
            return;
        }

        $this->_client = new PhpOrient($config['address'], 2424);
        $this->_client->username = $config['username'];
        $this->_client->password = $config['password'];

        // enable persistent connections
        $this->_client->setSessionToken(true);

        $db = stripos($config['address'], '172') !== false ? 'idoit' : 'test';
        $this->_client->dbOpen($db, 'root', 'netlog');
    }

    /**
     * Get least number of hops.
     *
     * @param  int $from
     * @param  int $to
     * @return array
     */
    public function get_least_number_of_hops($from, $to)
    {
        $sql = 'SELECT name, in_wire.@rid AS rids_in, out_wire.@rid AS rids_out FROM (SELECT expand(path) FROM (
  SELECT shortestPath($from, $to) AS path 
  LET 
    $from = (SELECT FROM bvg WHERE id='.$from.'), 
    $to = (SELECT FROM bvg WHERE id='.$to.') 
  UNWIND path
))';
        return $this->_execute_query($sql, $from);
    }

    /**
     * Get shortest path.
     *
     * @param  int $from
     * @param  int $to
     * @return array
     */
    public function get_shortest_path($from, $to)
    {
        $sql = 'SELECT name, in_wire.@rid AS rids_in, out_wire.@rid AS rids_out FROM (SELECT expand(path) FROM (
  SELECT dijkstra($from, $to, "length", "both") AS path 
  LET 
    $from = (SELECT FROM bvg WHERE id='.$from.'), 
    $to = (SELECT FROM bvg WHERE id='.$to.') 
  UNWIND path
))';
        return $this->_execute_query($sql, $from);
    }

    /**
     * Execute query and prepare result.
     *
     * @param  string $sql
     * @param  int $from
     * @return array
     */
    private function _execute_query($sql, $from)
    {
        $elements = $this->_client->query($sql);
        $path = [];
        $prev = $from;
        foreach ($elements as $l) {
            $d = $l->getOData();

            $wires = [];
            foreach ($d['rids_in'] as $rid) {
                $r = $rid->__toString();
                $wires[$r] = $r;
            }

            $path[] = array(
                'from'  => $prev,
                'to'    => $d['name'],
                'wires' => $wires,
            );
            $prev = $d['name'];
        }
        return $path;
    }

    /**
     * Synchronize connections between DB and OrientDB.
     */
    public function sync_connections()
    {
        $client = $this->_client;

        ini_set('memory_limit', '2048M');

        $dao = new \isys_cmdb_dao_smview_cable_connection_view(isys_application::instance()->database);
        $data = $dao->get_data();

        $lwls = $client->query( "select @rid, id, name from bvg limit 50000" );
        $lwl_ids = [];
        foreach ($lwls as $l) {
            $data1 = $l->getOData();
            $id = (int) $data1['id'];
            $rid = $data1['rid']->__toString();
            $lwl_ids[$id] = $rid;
        }

        $wires = $client->query( "select @rid, wire_id from wire limit 500000" );
        $wire_ids = [];
        foreach ($wires as $w) {
            $data1 = $w->getOData();
            $id = (int) $data1['wire_id'];
            $rid = $data1['rid']->__toString();
            $wire_ids[$id] = $rid;
        }

        foreach ($data as $d) {
            $in_id = (int) $d['obj_id_in'];
            $out_id = (int) $d['obj_id_out'];
            $length = (int) $d['length'];
            $length = $length == 0 ? 999999 : $length; // todo: use another placeholder
            $cable_id = (int) $d['cable_id'];
            $cable_title = $this->_escape_string($d['cable_title']);
            $wire_id = (int) $d['wire_id'];
            $wire_title = $this->_escape_string($d['wire_title']);

            $title = $this->_escape_string($d['obj_title_in']);
            if (! isset($lwl_ids[$in_id])) {
                $ret1 = $client->command( "insert into bvg set id = '$in_id', name = '$title'" );
                $rid = $ret1->getRid()->__toString();
                $lwl_ids[$in_id] = $rid;
            }

            $title = $this->_escape_string($d['obj_title_out']);
            if (! isset($lwl_ids[$out_id])) {
                $ret2 = $client->command( "insert into bvg set id = '$out_id', name = '$title'" );
                $rid = $ret2->getRid()->__toString();
                $lwl_ids[$out_id] = $rid;
            }

            if (! isset($wire_ids[$wire_id])) {
                $in = $lwl_ids[$in_id];
                $out = $lwl_ids[$out_id];
                $ret3 = $client->command( "create edge wire from $in to $out set distance = $length, wire_title='$wire_title', wire_id = $wire_id, cable_id = $cable_id, cable_title = '$cable_title'" );
                $rid = $ret3->getRid()->__toString();
                $wire_ids[$wire_id] = $rid;
            }
        }
    }

    /**
     * Escape some special characters.
     *
     * @param  string $string
     * @return string
     */
    private function _escape_string($string)
    {
        $string = str_replace("'", "", $string);
        $string = str_replace("\r", "", $string);
        $string = str_replace("\n", "", $string);
        return str_replace("\\", "", $string);
    }

    /**
     * Get cached issues.
     *
     * @param  string $conditions
     * @return array
     */
    public function get_issues($conditions = null)
    {
        if (empty($this->_client)) {
            return [];
        }

        $sql = "select * from Jira";
        if (isset($conditions)) {
            $condition_string = implode(' AND ', $conditions);
            $sql = "select * from Jira where $condition_string";
        }

        $current = $this->_client->command( $sql );
        $issues = [];
        foreach ($current as $l) {
            $data = $l->getOData();
            $id = $data['id'];

            // add "isys_" to each key
            $mod_data = [];
            foreach ($data as $k => $v) {
                $k = 'isys_' . $k;
                $mod_data[$k] = $v;
            }
            $mod_data['isys__id'] = $data['id'];
            $issues[$id] = $mod_data;
        }
        return $issues;
    }

    /**
     * Synchronize issues between JIRA and OrientDB.
     *
     * @param array $issues
     */
    public function sync_jira($issues)
    {
        if (empty($this->_client)) {
            return;
        }

        $current = $this->_client->command( "select * from Jira" );
        $current_keys = [];
        foreach ($current as $l) {
            $data1 = $l->getOData();
            $id = $data1['key'];
            $current_keys[$id] = $id;
        }

        $handled_keys = [];
        foreach ($issues as $id => $issue) {
            $key = $issue['key'];
            if (isset($current_keys[$key])) {
                // update?
                $handled_keys[$key] = $current_keys[$key];
                continue;
            }

            $props = [];
            foreach ($issue as $k => $v) {
                if (stripos($k, 'customfield_') === 0) {
                    // ignore custom fields
                    continue;
                }
                $value = $this->_escape_string($v);
                $props[] = "$k = '$value'";
            }
            $properties = implode(', ', $props);
            $sql = "insert into Jira set $properties";
            $this->_client->command($sql);
        }

        foreach ($current_keys as $key) {
            if (! isset($handled_keys[$key])) {
                // remove obsolete issue
                $this->_client->command( "delete from Jira where key = '$key'" );
            }
        }
    }

    /**
     * Get all fields of an issue.
     *
     * @return array
     */
    public function get_issue_fields()
    {
        if (empty($this->_client)) {
            return [];
        }

        $sql = "select * from JiraProperties";
        $currents = $this->_client->command( $sql );
        $fields = [];
        foreach ($currents as $current) {
            $data = $current->getOData();
            $fields[$data['id']] = $data;
        }
        return $fields;
    }

    /**
     * Synchronize fields of an issue between JIRA and OrientDB.
     *
     * @param array $fields
     */
    public function sync_issue_fields($fields)
    {
        if (empty($this->_client)) {
            return;
        }

        $current = $this->_client->command( "select * from JiraProperties" );
        $current_keys = [];
        foreach ($current as $l) {
            $data1 = $l->getOData();
            $id = $data1['id'];
            $current_keys[$id] = $id;
        }

        $handled_keys = [];
        foreach ($fields as $id => $field) {
            $key = $field['id'];
            if (isset($current_keys[$key])) {
                // update?
                $handled_keys[$key] = $current_keys[$key];
                continue;
            }

            $props = [];
            foreach ($field as $k => $v) {
                $value = $this->_escape_string($v);
                $props[] = "$k = '$value'";
            }
            $properties = implode(', ', $props);
            $sql = "insert into JiraProperties set $properties";
            $this->_client->command($sql);
        }

        foreach ($current_keys as $key) {
            if (! isset($handled_keys[$key])) {
                // remove obsolete issue
                $this->_client->command( "delete from JiraProperties where key = '$key'" );
            }
        }
    }
}
