<?php

namespace idoit\Module\Feed\DependentObjectManager;

use Exception;
use idoit\Module\Feed\ProxyApi\Idoit;
use isys_application;
use isys_cmdb_dao_smview_feed_idoit;
use isys_exception_api;
use isys_exception_database;
use isys_helper_link;
use isys_module_smview_feed;

/**
 * Idoit Manager class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class IdoitManager extends AbstractManager
{

    /**
     * Property mapping.
     *
     * @var array
     */
    protected $_mapping = array(
        'name'                 => 'C__CATG__SMVIEW_FEED_NAME',
        'address'              => 'C__CATG__SMVIEW_FEED_URL',
        'key'                  => 'C__CATG__SMVIEW_FEED_KEY',
        'username'             => 'C__CATG__SMVIEW_FEED_USERNAME',
        'password'             => 'C__CATG__SMVIEW_FEED_PASSWORD',
        'indexed_object_types' => 'C__CATG__SMVIEW_FEED_INDEXED_OBJECT_TYPES',
    );

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->_dao = new isys_cmdb_dao_smview_feed_idoit(isys_application::instance()->database);
    }

    /**
     * Process generator listing.
     * @throws isys_exception_database
     * @throws Exception
     */
    public function processListing()
    {
        $lang = isys_application::instance()->container->get('language');
        $headers = [
            "id"                      => "ID",
            "name"                    => $lang->get('LC_UNIVERSAL__NAME'),
            "address"                 => $lang->get('LC__CATG__IP_ADDRESS'),
            "username"                => $lang->get('LC__LOGIN__USERNAME'),
            "last_object_update"      => $lang->get('LC__CMDB__CATG__SMVIEW_FEED__IDOIT_LAST_OBJECT_UPDATE'),
            "last_object_type_update" => $lang->get('LC__CMDB__CATG__SMVIEW_FEED__IDOIT_LAST_OBJECT_TYPE_UPDATE'),
        ];

        $res = $this->_dao->get_objects();
        $instances = [];
        foreach ($res as $r) {
            try {
                $proxy_api = new Idoit();
                $i = $proxy_api->get_instance($r['id']);

                $time = strtotime($i['object_update_status']['last_finished']);
                $r['last_object_update'] = date('d.m.Y h:i:s P', $time);
                $time = strtotime($i['object_type_update_status']['last_finished']);
                $r['last_object_type_update'] = date('d.m.Y h:i:s P', $time);
            } catch (isys_exception_api $e) {
            } catch (isys_exception_database $e) {
            }
            $instances[] = $r;
        }

        $title = $lang->get('LC__CMDB__SMVIEW_FEED__ADAPTER_IDOIT');
        $this->_processListing($title, $headers, $instances);
    }

    /**
     * Process page.
     *
     * @param  int $element_id
     * @param  array $additional_fields
     * @throws isys_exception_database
     */
    public function processPage($element_id, $additional_fields = [])
    {
        $additional_fields = $this->_getAdditionalFields($element_id);
        parent::processPage($element_id, $additional_fields);
    }

    /**
     * Process new page.
     *
     * @param  array $additional_fields
     * @throws isys_exception_database
     */
    public function processNewPage($additional_fields = [])
    {
        $additional_fields = $this->_getAdditionalFields();
        parent::processNewPage($additional_fields);
    }

    /**
     * Add list data to rules array.
     *
     * @param  array $l_rules
     * @return array
     */
    protected function _add_listing_rules($l_rules)
    {
        $l_rules['C__CATG__SMVIEW_FEED_URL']['p_strPlaceholder'] = 'http(s)://192.168.1.2/i-doit/src/jsonrpc.php';
        return $l_rules;
    }

    /**
     * Get additional field information.
     *
     * @param  int|null $element_id
     * @return array
     * @throws isys_exception_database
     */
    private function _getAdditionalFields($element_id = null)
    {
        $additional_fields = [
            'assets_dir'  => isys_module_smview_feed::get_assets_www_dir(),
            'spinner_url' => isys_helper_link::get_base() . 'images/ajax-loading.gif',
        ];

        $lang = isys_application::instance()->container->get('language');
        $js_config = [
            'object_types' => $this->_getObjectTypes($element_id),
            'language'    => [
                'LC_UNIVERSAL__OBJECT_TYPE' => $lang->get('LC_UNIVERSAL__OBJECT_TYPE'),
            ],
            'is_editable' => isys_glob_is_edit_mode(),
        ];
        $additional_fields['js_config'] = json_encode($js_config);
        if (empty($element_id)) {
            return $additional_fields;
        }

        try {
            $proxy_api = new Idoit();
            $i = $proxy_api->get_instance($element_id);
            $time = strtotime($i['object_update_status']['last_finished']);
            $additional_fields['last_object_update'] = date('d.m.Y h:i:s', $time);
            $time = strtotime($i['object_type_update_status']['last_finished']);
            $additional_fields['last_object_type_update'] = date('d.m.Y h:i:s', $time);
            $errorMessage = $i['last_error_message'];
            $errorMessage = strlen($errorMessage) > 2500 ? substr($errorMessage, 0, 2500) . ' [...]' : $errorMessage;
            $additional_fields['last_error'] = $errorMessage;
        } catch (isys_exception_api $e) {
        } catch (isys_exception_database $e) {
        }

        $additional_fields['ajax_url'] = isys_helper_link::create_url([
            C__GET__AJAX      => 1,
            C__GET__AJAX_CALL => 'feed_config',
            'func'            => 'reindex-idoit',
            'instance_id'     => $element_id]);
        return $additional_fields;
    }

    /**
     * Get all object types and check selected ones.
     *
     * @param  int $element_id
     * @return array
     * @throws isys_exception_database
     */
    private function _getObjectTypes($element_id = null)
    {
        // get indexed object types
        $listing = [];
        if ($element_id) {
            $obj = $this->_dao->get_object('id', $element_id);
            $str = $obj['indexed_object_types'];
            $listing = explode(';', $str);
        }

        $objectTypes = [];
        $lang = isys_application::instance()->container->get('language');
        $res = $this->_dao->retrieve('SELECT * from isys_obj_type');
        while ($row = $res->get_row()) {
            $title = $lang->get($row['isys_obj_type__title']);
            $objectTypes[$title] = [
                'id'          => $row['isys_obj_type__id'],
                'title'       => $title,
                'const'       => $row['isys_obj_type__const'],
                'is_selected' => in_array($row['isys_obj_type__const'], $listing),
            ];
        }
        ksort($objectTypes);
        return array_values($objectTypes);
    }
}
