<?php

namespace idoit\Module\Feed\JiraApi;

use isys_application;
use isys_cache_memcached;
use isys_cmdb_dao_smview_feed_jira;
use isys_exception_api;
use isys_exception_database;
use JiraRestApi\Configuration\ArrayConfiguration;
use JiraRestApi\Field\FieldService;
use JiraRestApi\Issue\Issue;
use JiraRestApi\Issue\IssueField;
use JiraRestApi\Issue\IssueService;
use JiraRestApi\Issue\IssueStatus;
use JiraRestApi\Issue\IssueType;
use JiraRestApi\Issue\Priority;
use JiraRestApi\Issue\Reporter;
use JiraRestApi\JiraException;
use JiraRestApi\Project\Project;
use JsonMapper_Exception;

/**
 * JIRA API class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Jira
{

    /**
     * Cache connector timeout.
     */
    const CLIENT_TIMEOUT = 8;

    /**
     * Maximum retries to request cache.
     */
    const MAX_CACHE_RETRIES = 10;

    /**
     * Cache request delay in seconds.
     */
    const CACHE_REQUEST_DELAY = 1;

    /**
     * Field name separator to replace '.' notation.
     */
    const FIELD_NAME_SEPARATOR = '___';

    /**
     * Get all issues from JIRA instance.
     *
     * @param int        $instance_id
     * @param array|null $conditions
     *
     * @return array
     * @throws JiraException
     * @throws JsonMapper_Exception
     * @throws isys_exception_database
     */
    public function get_issues_from_instance(int $instance_id, ?array $conditions = null): array
    {
        $identifier = $instance_id . '-' . implode(',', $conditions);
        $identifier = str_replace("'", "", $identifier);
        $identifier = str_replace(" ", "", $identifier);

        $m_cache = new isys_cache_memcached();
        if (isys_cache_memcached::available()) {
            $objects = $m_cache->get($identifier);
        }

        if (empty($objects)) {
            $config = $this->_getConfigByInstanceId($instance_id);
            $issueService = new IssueService($config);

            $newConditions = [];
            foreach ($conditions as $c) {
                $n = str_replace(':', '=', $c);
                $n = str_replace('fields.', '', $n);
                $newConditions[] = $n;
            }

            try {
                $query_string = implode(' AND ', $newConditions);
                $ret = $issueService->search($query_string);
            } catch (JiraException $e) {
                return [];
            }

            $objects = $ret->issues;
            if (isys_cache_memcached::available()) {
                $m_cache->set($identifier, $objects, 30);
            }
        }

        $issues = [];
        foreach ($objects as $issue) {
            $issues[$issue->id] = $this->_serialize_issue($issue);
        }
        return $issues;
    }

    /**
     * Get JIRA configuration by instance ID.
     *
     * @param  int $instance_id
     *
     * @return ArrayConfiguration
     * @throws isys_exception_database
     */
    private function _getConfigByInstanceId(int $instance_id): ArrayConfiguration
    {
        $dao = new isys_cmdb_dao_smview_feed_jira(isys_application::instance()->database);
        $jira_server = $dao->get_object('id', $instance_id);

        return new ArrayConfiguration([
            'jiraHost' => $jira_server['address'],

            // basic authentication (deprecated)
            'jiraUser' => $jira_server['username'],
            'jiraPassword' => $jira_server['password'],

            // token based authentication
//            'useTokenBasedAuth' => true,
//            'personalAccessToken' => '',

            // custom log config
            'jiraLogEnabled' => true,
            'jiraLogFile' => "/var/www/html/i-doit/temp/jira.log",
            'jiraLogLevel' => 'INFO',
        ]);
    }

    /**
     * Serialize issue.
     *
     * @param  Issue $issue
     * @return array
     */
    private function _serialize_issue(Issue $issue): array
    {
        $data = [];
        $fields = $issue->fields;
        foreach ($fields as $id => $field) {
            $value = $field;
            if (is_array($field)) {
                if (is_array($field[0])) {
                    // handle array of nested elements
                    $values = [];
                    foreach ($field as $f) {
                        if ($f['name']) {
                            // consider only "name" property of a nested structure
                            $values[] = $f['name'];
                        }
                    }
                    $value = implode(', ', $values);
                } else {
                    if (! $field['name']) {
                        // consider only "name" property of a nested structure
                        continue;
                    }
                    $value = $field['name'];
                }
            }


            if ($field instanceof IssueType || $field instanceof Priority || $field instanceof IssueStatus || $field instanceof Project) {
                if (! $field->name) {
                    // consider only "name" property of a nested structure
                    continue;
                }
                $value = $field->name;
            }

            if ($field instanceof Reporter) {
                $array = $field->toArray();
                if (! $array['emailAddress']) {
                    continue;
                }
                $value = $field->emailAddress;
            }

            if ($field instanceof Issue) {
                if (! $field->key) {
                    // consider only "name" property of a nested structure
                    continue;
                }
                $value = $field->key;
            }

            if ($field instanceof \DateTime) {
                $value = $field->getTimestamp();
            }

            if ($field instanceof \stdClass) {
                continue;
            }

            // add "isys_" to each key
            $id = 'isys_' . $id;
            $data[$id] = $value;
        }

        // rename ID
        $data['isys__id'] = $issue->id;

        // add special fields
        $data['isys_key'] = $issue->key;
        $data['isys_self'] = $issue->self;
        return $data;
    }

    /**
     * Get all fields of an issue.
     *
     * @param int $instance_id
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_fields_from_instance(int $instance_id): array
    {
        $identifier = $instance_id . '-' . 'fields';
        $m_cache = new isys_cache_memcached();
        if (isys_cache_memcached::available() && $m_cache->exists($identifier)) {
            return $m_cache->get($identifier);
        }

        $config = $this->_getConfigByInstanceId($instance_id);
        try {
            $fieldService = new FieldService($config);
            $data = $fieldService->getAllFields();
        } catch (JiraException $e) {
            return [];
        }

        $storage = $data->getArrayCopy();
        $fields = [];
        foreach ($storage as $m) {
            $fields[$m->id] = [
                'id'   => $m->id,
                'name' => $m->name,
                'type' => $m->schema->type,
            ];
        }

        if (isys_cache_memcached::available()) {
            $m_cache->set($identifier, $fields, 30);
        }
        return $fields;
    }

    /**
     * @param  int $obj_id
     * @param  string $obj_title
     * @param  string|null $obj_description
     * @param  string|null $author
     *
     * @throws JiraException
     * @throws JsonMapper_Exception
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function create_jira_ticket(int $obj_id, string $obj_title, ?string $obj_description = null, ?string $author = null)
    {
        $dao = new isys_cmdb_dao_smview_feed_jira(isys_application::instance()->database);
        $jira_servers = $dao->get_objects();
        $jira_server = $jira_servers[0];
        $config = $this->_getConfigByInstanceId($jira_server['id']);

        if (empty($obj_description)) {
            $dao = new \isys_cmdb_dao_category_g_global(isys_application::instance()->database);
            $obj = $dao->get_data(null, $obj_id)->get_row();
            $obj_description = strip_tags($obj['isys_obj__description']);
        }

        if (empty($author)) {
            $userDao = \isys_component_dao_user::instance(isys_application::instance()->database);
            $userId = $userDao->get_current_user_id();
            $author = $userDao->get_user_title($userId);
        }

        // create issue
        $issueService = new IssueService($config);
        try {
            $issueField = new IssueField();
            $issueField->setProjectKey("INFRA400")
                ->setSummary($obj_description)
                ->setAssigneeNameAsString($author)
                ->setPriorityNameAsString("Medium")
                ->setIssueTypeAsString("Bug")
                ->setDescription("Dies ist ein generiertes Test-Ticket")
                ->addComponentsAsArray([$obj_title]);
            $issueService->create($issueField);
        } catch (JiraException $e) {
            throw new isys_exception_api($e->getMessage());
        }
    }
}
