<?php

namespace idoit\Module\Feed\ProxyApi;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use isys_application;
use isys_cmdb_dao_smview_feed_configuration;
use isys_exception_api;
use isys_exception_database;
use Psr\Http\Message\ResponseInterface;

/**
 * Proxy API class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class AbstractApi
{

    /**
     * Cache connector timeout.
     */
    const CLIENT_TIMEOUT = 8;

    /**
     * Maximum retries to request cache.
     */
    const MAX_CACHE_RETRIES = 10;

    /**
     * Cache request delay in seconds.
     */
    const CACHE_REQUEST_DELAY = 1;

    /**
     * URI of authentication token.
     */
    const AUTH_TOKEN_URI = '/token';

    /**
     * Cache proxy address.
     *
     * @var string|null
     */
    protected $_proxy_address = null;

    /**
     * Current authentication token.
     *
     * @var string|null
     */
    protected $_token = null;

    /**
     * Constructor.
     *
     * @throws isys_exception_database
     * @throws isys_exception_api
     */
    public function __construct()
    {
        // get configured cache
        $dao = new isys_cmdb_dao_smview_feed_configuration(isys_application::instance()->database);
        $config = $dao->get_object('id', 1);

        if (empty($config['address'])) {
            // todo: handle error
            return;
        }
        $this->_token = $this->_get_auth_token($config);
        $this->_proxy_address = $config['address'] . '/api/v1/jiras/';
    }

    /**
     * Get authentication token.
     *
     * @param  array $config
     * @return string
     * @throws isys_exception_api
     */
    private function _get_auth_token($config)
    {
        $uri = $config['address'] . self::AUTH_TOKEN_URI;
        $params = [
            'username' => $config['username'],
            'password' => $config['password'],
        ];
        $token = $this->_request($uri, 'POST', null, $params);
        if (! $token) {
            throw new isys_exception_api('could not get authentication token');
        }
        return $token['token_type'] . ' ' . $token['access_token'];
    }

    /**
     * Request given URI.
     *
     * @param  string $uri
     * @param  string $method
     * @param  array $params
     * @param  array $form_params
     * @return mixed|ResponseInterface
     * @throws isys_exception_api
     */
    protected function _request($uri, $method = 'GET', $params = [], $form_params = [])
    {
        $client = new Client(['timeout' => self::CLIENT_TIMEOUT]);
        $timeout = time() + (self::CACHE_REQUEST_DELAY * self::MAX_CACHE_RETRIES);
        $last_response = null;
        while (time() < $timeout) {
            $requestOptions = [];
            if ($this->_token) {
                $requestOptions['headers'] = [
                    'Authorization' => $this->_token
                ];
            }
            try {
                if ($method == 'POST' || $method == 'PUT') {
                    if ($params) {
                        $requestOptions['body'] = json_encode($params);
                    }
                    if ($form_params) {
                        $requestOptions['form_params'] = $form_params;
                    }
                    $response = $client->request($method, $uri, $requestOptions);
                } else {
                    $response = $client->request($method, $uri, $requestOptions);
                }
            } catch(GuzzleException $e) {
                if ($e->getCode() == 404) {
                    // requested item not found
                    return [];
                }

                // parse error message
                $message = $e->getMessage();
                $messageParts = explode("\n", $e->getMessage());
                foreach ($messageParts as $m) {
                    $jsonMsg = json_decode($m, true);
                    if ($jsonMsg) {
                        $messages = [];
                        $details = $jsonMsg['detail'];
                        foreach ($details as $d) {
                            $msg = implode(' -> ', $d['loc']);
                            $msg .= ': ' . $d['msg'] . ' (' . $d['type'] . ')';
                            $messages[] = $msg;
                        }
                        $message = implode("\n", $messages);
                    }
                }
                throw new isys_exception_api($message, $e->getCode());
            }

            if ($response->getStatusCode() == 202) {
                // wait
                $last_response = $this->_handle_response($response);
                sleep(self::CACHE_REQUEST_DELAY);
            } else {
                return $this->_handle_response($response);
            }
        }
        return ! empty($last_response) ? $last_response : [];
    }

    /**
     * Handle response.
     *
     * @param  mixed|ResponseInterface $response
     * @return array
     * @throws isys_exception_api
     */
    private function _handle_response($response)
    {
        $valid_status_codes = [200, 201, 202];
        if (! in_array($response->getStatusCode(), $valid_status_codes) || ! $response->getBody()) {
            $message = 'Request returned an error with code: ' . $response->getStatusCode();
            throw new isys_exception_api($message, $response->getStatusCode());
        }

        $json = '';
        while ($part = $response->getBody()->read(512)) {
            $json .= $part;
        }

        return json_decode($json, true);
    }
}
