<?php

namespace idoit\Module\Feed\ProxyApi;

use isys_application;
use isys_cache_memcached;
use isys_cmdb_dao_smview_feed_idoit;
use isys_exception_api;
use isys_exception_database;

/**
 * Proxy API class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Idoit extends AbstractApi
{

    /**
     * Field name separator to replace '.' notation.
     */
    const FIELD_NAME_SEPARATOR = '___';

    /**
     * List of known Idoit parameters.
     *
     * @var array
     */
    private $_known_idoit_params = ['name', 'url', 'key', 'username', 'password', 'indexed_object_types'];

    /**
     * Constructor.
     *
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function __construct()
    {
        parent::__construct();
        $this->_proxy_address = str_replace('/api/v1/jiras/', '/api/v1/idoit/', $this->_proxy_address);
    }

    /**
     * Get Idoit URL by ID.
     *
     * @param  int $instance_id
     * @return string
     * @throws isys_exception_database
     */
    private function _get_idoit_instance_name($instance_id)
    {
        $dao = new isys_cmdb_dao_smview_feed_idoit(isys_application::instance()->database);
        $idoit_server = $dao->get_object('id', $instance_id);
        return $idoit_server['name'];
    }

    /**
     * Get all issues of all instances.
     *
     * @param  array|null $conditions
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function get_all_objects($conditions = null)
    {
        $dao = new isys_cmdb_dao_smview_feed_idoit(isys_application::instance()->database);
        $idoit_servers = $dao->get_objects();
        $objects = [];
        foreach ($idoit_servers as $idoit_server) {
            $i = $this->get_objects_from_instance($idoit_server['id'], $conditions);
            foreach ($i as $obj_id => $s) {
                // merge objects per instance
                if ($objects[$obj_id]) {
                    $objects[$obj_id] = array_merge($objects[$obj_id], $s);
                } else {
                    $objects[$obj_id] = $s;
                }
            }
        }
        return $objects;
    }

    /**
     * Get all objects from Idoit instance.
     *
     * @param  int $instance_id
     * @param  array|null $conditions
     * @return array
     * @throws isys_exception_api
     */
    public function get_objects_from_instance($instance_id, $conditions = null)
    {
        if (empty($this->_proxy_address)) {
            return [];
        }

        $identifier = $instance_id . '-' . implode(',', $conditions);
        $identifier = str_replace("'", "", $identifier);
        $identifier = str_replace(" ", "", $identifier);

        $m_cache = new isys_cache_memcached();
        if (isys_cache_memcached::available()) {
            $objects = $m_cache->get($identifier);
        }

        if (empty($objects)) {
            $objects = [];
            $offset = 0;
            $query_string = implode(' AND ', $conditions);
            while(true) {
                $uri = $this->_proxy_address . "objects/?limit=1000&offset=$offset&query=" . urlencode($query_string);
                $listing = $this->_request($uri);
                if (empty($listing)) {
                    break;
                }
                $objects = array_merge($objects, $listing);
                if (count($objects) < 1000) {
                    break;
                }
                $offset += 1000;
            }
            if (isys_cache_memcached::available()) {
                $m_cache->set($identifier, $objects, 30);
            }
        }

        $issues = [];
        foreach ($objects as $object) {
            $issues[$object['id']] = $this->_serialize_object($object);
        }
        return $issues;
    }

    /**
     * Serialize object.
     *
     * @param  array $object
     * @return array
     */
    private function _serialize_object($object)
    {
        $data = [];
        foreach ($object as $id => $field) {
            $value = $field;
            if (is_array($field)) {
                if (! $field['name']) {
                    // consider only "name" property of a nested structure
                    continue;
                }
                $value = $field['name'];
                $id .= self::FIELD_NAME_SEPARATOR . 'name';
            }
            $data[$id] = $value;
        }
        return $data;
    }

    /**
     * Get Idoit instance by ID.
     *
     * @param  int $instance_id
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function get_instance($instance_id)
    {
        if (empty($this->_proxy_address)) {
            return [];
        }

        $uri = $this->_proxy_address . $this->_get_idoit_instance_name($instance_id);
        return $this->_request($uri);
    }

    /**
     * Create new Idoit instance.
     *
     * @param  array $data
     * @return array
     * @throws isys_exception_api
     */
    public function create_instance($data)
    {
        $params = [
            'object_update_status' => [
                'interval_sec' => 600
            ],
            'object_type_update_status' => [
                'interval_sec' => 300
            ]
        ];
        foreach ($this->_known_idoit_params as $p) {
            if (! empty($data[$p])) {
                $params[$p] = $data[$p];
            }
        }
        return $this->_request($this->_proxy_address, 'POST', $params);
    }

    /**
     * Update existing Idoit instance.
     *
     * @param  int $instance_id
     * @param  array $data
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function update_instance($instance_id, $data)
    {
        $uri = $this->_proxy_address . $this->_get_idoit_instance_name($instance_id);
        $params = [];
        foreach ($this->_known_idoit_params as $p) {
            if (! empty($data[$p])) {
                $params[$p] = $data[$p];
            }
        }
        return $this->_request($uri, 'PUT', $params);
    }

    /**
     * Re-Index existing Idoit instance.
     *
     * @param  int $instance_id
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function re_index_instance($instance_id)
    {
        // delete whole index
        $this->delete_instance($instance_id);

        // add this index again
        $dao = new isys_cmdb_dao_smview_feed_idoit(isys_application::instance()->database);
        $idoit_server = $dao->get_object('id', $instance_id);
        unset($idoit_server['id']);
        $idoit_server['url'] = $idoit_server['address'];
        $this->create_instance($idoit_server);
    }

    /**
     * Delete Idoit instance.
     *
     * @param  int $instance_id
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function delete_instance($instance_id)
    {
        $uri = $this->_proxy_address . $this->_get_idoit_instance_name($instance_id);
        return $this->_request($uri, 'DELETE');
    }
}
