<?php

namespace idoit\Module\Feed\ProxyApi;

use isys_application;
use isys_cache_memcached;
use isys_cmdb_dao_smview_feed_jira;
use isys_exception_api;
use isys_exception_cache;
use isys_exception_database;

/**
 * Proxy API class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Jira extends AbstractApi
{

    /**
     * Field name separator to replace '.' notation.
     */
    const FIELD_NAME_SEPARATOR = '___';

    /**
     * List of known Jira parameters.
     *
     * @var array
     */
    private $_known_jira_params = ['url', 'name', 'username', 'password'];

    /**
     * Get Jira URL by ID.
     *
     * @param  int $instance_id
     * @return string
     * @throws isys_exception_database
     */
    private function _get_jira_instance_name($instance_id)
    {
        $dao = new isys_cmdb_dao_smview_feed_jira(isys_application::instance()->database);
        $jira_server = $dao->get_object('id', $instance_id);

        return $jira_server['name'];
    }

    /**
     * Get all issues of all instances.
     *
     * @param  array|null $conditions
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function get_all_issues($conditions = null)
    {
        $dao = new isys_cmdb_dao_smview_feed_jira(isys_application::instance()->database);
        $jira_servers = $dao->get_objects();
        $issues = [];
        $jiraApi = new \idoit\Module\Feed\JiraApi\Jira();
        foreach ($jira_servers as $jira_server) {
            if ($jira_server['use_api_directly']) {
                $i = $jiraApi->get_issues_from_instance($jira_server['id'], $conditions);
            } else {
                $i = $this->get_issues_from_instance($jira_server['id'], $conditions);
            }
            foreach ($i as $obj_id => $s) {
                // merge issues per object
                if ($issues[$obj_id]) {
                    $issues[$obj_id] = array_merge($issues[$obj_id], $s);
                } else {
                    $issues[$obj_id] = $s;
                }
            }
        }
        return $issues;
    }

    /**
     * Get all issues from JIRA instance.
     *
     * @param  int $instance_id
     * @param  array|null $conditions
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function get_issues_from_instance($instance_id, $conditions = null)
    {
        if (empty($this->_proxy_address)) {
            return [];
        }

        // add instance to query
        $instance_name = $this->_get_jira_instance_name($instance_id);
        $conditions[] = "jira: $instance_name";

        $identifier = $instance_id . '-' . implode(',', $conditions);
        $identifier = str_replace("'", "", $identifier);
        $identifier = str_replace(" ", "", $identifier);

        $m_cache = new isys_cache_memcached();
        if (isys_cache_memcached::available()) {
            $objects = $m_cache->get($identifier);
        }

        if (empty($objects)) {
            $objects = [];
            $offset = 0;
            $query_string = implode(' AND ', $conditions);
            while(true) {
                $uri = $this->_proxy_address . "issues/?limit=1000&offset=$offset&query=" . urlencode($query_string);
                $listing = $this->_request($uri);
                if (empty($listing)) {
                    break;
                }
                $objects = array_merge($objects, $listing);
                if (count($objects) < 1000) {
                    break;
                }
                $offset += 1000;
            }
            if (isys_cache_memcached::available()) {
                $m_cache->set($identifier, $objects, 30);
            }
        }

        $issues = [];
        foreach ($objects as $object) {
            $issues[$object['id']] = $this->_serialize_issue($object);
        }
        return $issues;
    }

    /**
     * Serialize issue.
     *
     * @param  array $issue
     * @return array
     */
    private function _serialize_issue($issue)
    {
        $data = [];
        $fields = $issue['fields'];
        foreach ($fields as $id => $field) {
            $value = $field;
            if (is_array($field)) {
                if (is_array($field[0])) {
                    // handle array of nested elements
                    $values = [];
                    foreach ($field as $f) {
                        if ($f['name']) {
                            // consider only "name" property of a nested structure
                            $values[] = $f['name'];
                        }
                    }
                    $value = implode(', ', $values);
                } else {
                    if (! $field['name']) {
                        // consider only "name" property of a nested structure
                        continue;
                    }
                    $value = $field['name'];
                }
                $id .= self::FIELD_NAME_SEPARATOR . 'name';
            }

            // add "isys_" to each key
            $id = 'isys_' . $id;
            $data[$id] = $value;
        }

        // rename ID
        $data['isys__id'] = $issue['id'];

        // add special fields
        $data['isys_key'] = $issue['key'];
        $data['isys_jira'] = $issue['jira'];
        $data['isys_self'] = $issue['self'];
        return $data;
    }

    /**
     * Get all fields of all instances.
     *
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function get_all_fields()
    {
        $dao = new isys_cmdb_dao_smview_feed_jira(isys_application::instance()->database);
        $jira_servers = $dao->get_objects();
        $fields = [];
        $jiraApi = new \idoit\Module\Feed\JiraApi\Jira();
        foreach ($jira_servers as $jira_server) {
            if ($jira_server['use_api_directly']) {
                $i = $jiraApi->get_fields_from_instance($jira_server['id']);
            } else {
                $i = $this->get_fields_from_instance($jira_server['id']);
            }
            foreach ($i as $obj_id => $s) {
                // merge fields per object
                if ($fields[$obj_id]) {
                    $fields[$obj_id] = array_merge($fields[$obj_id], $s);
                } else {
                    $fields[$obj_id] = $s;
                }
            }
        }
        return $fields;
    }

    /**
     * Get all fields of an issue.
     *
     * @param int $instance_id
     * @return array
     * @throws isys_exception_database
     * @throws isys_exception_api
     */
    public function get_fields_from_instance($instance_id)
    {
        if (empty($this->_proxy_address)) {
            return [];
        }

        $uri = $this->_proxy_address . $this->_get_jira_instance_name($instance_id) . "/fields/";
        $data = $this->_request($uri);
        $field_data = $data['properties']['fields']['properties'];
        $meta_data = $data['_meta'];
        $field_names = [];
        foreach ($meta_data as $m) {
            $field_names[$m['id']] = $m['name'];
        }

        $fields = [];
        foreach ($field_data as $id => $field) {
            $full_id = $id;
            $type = null;
            if ($field['properties']) {
                // nested field
                $nestedFields = $field['properties'];
                if (! $nestedFields['name']) {
                    // consider only "name" property of a nested structure
                    continue;
                }
                $full_id = $id . self::FIELD_NAME_SEPARATOR . 'name';
                $type = $nestedFields['type'];
            } else {
                $type = $field['type'];
            }
            $fields[$full_id] = [
                'id'   => $full_id,
                'name' => isset($field_names[$id]) ? $field_names[$id] : $id,
                'type' => $type,
            ];
        }
        return $fields;
    }

    /**
     * Get Jira instance by ID.
     *
     * @param  int $instance_id
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function get_instance($instance_id)
    {
        if (empty($this->_proxy_address)) {
            return [];
        }

        $uri = $this->_proxy_address . $this->_get_jira_instance_name($instance_id);
        return $this->_request($uri);
    }

    /**
     * Create new Jira instance.
     *
     * @param  array $data
     * @return array
     * @throws isys_exception_api
     */
    public function create_instance($data)
    {
        $params = [
            'options' => [
                'verify' => false,
                'timeout' => 1,
            ],
            'cache_timeout_sec' => 300,
        ];
        foreach ($this->_known_jira_params as $p) {
            if (! empty($data[$p])) {
                $params[$p] = $data[$p];
            }
        }
        return $this->_request($this->_proxy_address, 'POST', $params);
    }

    /**
     * Update existing Jira instance.
     *
     * @param  int $instance_id
     * @param  array $data
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function update_instance($instance_id, $data)
    {
        $uri = $this->_proxy_address . $this->_get_jira_instance_name($instance_id);
        $params = [];
        foreach ($this->_known_jira_params as $p) {
            if (! empty($data[$p])) {
                $params[$p] = $data[$p];
            }
        }
        return $this->_request($uri, 'PUT', $params);
    }

    /**
     * Re-Index existing Jira instance.
     *
     * @param  int $instance_id
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function re_index_instance($instance_id)
    {
        // delete whole index
        $this->delete_instance($instance_id);

        // add this index again
        $dao = new isys_cmdb_dao_smview_feed_jira(isys_application::instance()->database);
        $jira_server = $dao->get_object('id', $instance_id);
        unset($jira_server['id']);
        $jira_server['url'] = $jira_server['address'];
        $this->create_instance($jira_server);
    }

    /**
     * Delete Jira instance.
     *
     * @param  int $instance_id
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function delete_instance($instance_id)
    {
        $uri = $this->_proxy_address . $this->_get_jira_instance_name($instance_id);
        return $this->_request($uri, 'DELETE');
    }
}
