<?php

namespace idoit\Module\Feed\ProxyApi;

use Exception;
use isys_application;
use isys_exception_api;
use isys_exception_database;

/**
 * Proxy API class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class VirtualDevice extends AbstractApi
{

    /**
     * List of known parameters with its language templates.
     *
     * @var array
     */
    private $_known_params = [
        'user_name'   => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_USER_NAME',
        'mac_address' => 'LC__CMDB__CATG__PORT__MAC',
        'host_name'   => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_HOST_NAME',
        'ip_address'  => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_IP_ADDRESS_END_DEVICE',
        'subnet'      => 'LC__CMDB__CATS__NET_IP_ADDRESSES__SUBNET',
        'device_name' => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_DEVICE_NAME',
        'device'      => 'LC__CATP__IP__ADDRESS',
        'port'        => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_PORT',
        'port_name'   => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_PORT_NAME',
        'port_state'  => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_PORT_STATE',
        'vtp_domain'  => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_VTP_DOMAIN',
        'vlan'        => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_VLAN',
        'vlan_type'   => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_VLAN_TYPE',
        'last_seen'   => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_LAST_SEEN',
        'notes'       => 'LC__CMDB__CATG__SMVIEW_FEED__VIRTUAL_DEVICE_NOTES',
    ];

    /**
     * Constructor.
     *
     * @throws isys_exception_database
     * @throws isys_exception_api
     */
    public function __construct()
    {
        parent::__construct();
        $this->_proxy_address = str_replace('/api/v1/jiras/', '/api/v1/macmon/', $this->_proxy_address);
    }

    /**
     * Get all fields.
     *
     * @return array
     * @throws Exception
     */
    public function get_all_fields()
    {
        $language = isys_application::instance()->container->get('language');
        $fields = [];
        foreach ($this->_known_params as $param => $label) {
            $fields[] = [
                'id'   => $param,
                'name' => $language->get($label),
                'type' => 'string'
            ];
        }
        return $fields;
    }

    /**
     * Get devices.
     *
     * @return array
     * @throws isys_exception_api
     */
    public function get_devices()
    {
        $devices = [];
        if (empty($this->_proxy_address)) {
            return [];
        }

        $uri = $this->_proxy_address;
        $objects = $this->_request($uri);
        foreach ($objects as $object) {
            $devices[$this->_getId($object['mac_address'])] = $this->_serialize_device($object);
        }
        return $devices;
    }

    /**
     * Query devices by conditions.
     *
     * @param  array $conditions
     * @param  int $limit
     * @param  int|null $offset
     * @return array
     * @throws isys_exception_api
     */
    public function query_devices($conditions, $limit = 100, $offset = null)
    {
        $devices = [];
        if (empty($this->_proxy_address)) {
            return [];
        }

        $query = implode(' AND ', $conditions);
        $query = str_replace("'", '"', $query);
        $query = urlencode($query);
        $uri = $this->_proxy_address . "?query=$query&limit=$limit";
        if (isset($offset)) {
            $uri .= "&offset=$offset";
        }
        $objects = $this->_request($uri);
        foreach ($objects as $object) {
            $devices[$this->_getId($object['mac_address'])] = $this->_serialize_device($object);
        }
        return $devices;
    }

    /**
     * Serialize device.
     *
     * @param  array $device
     * @return array
     */
    private function _serialize_device($device)
    {
        $data = [];
        foreach ($device as $id => $field) {
            // add "isys_" to each key
            $id = 'isys_' . $id;
            $data[$id] = $field;
        }

        // rename ID
        $data['isys__id'] = $this->_getId($device['mac_address']);
        return $data;
    }

    /**
     * Create an ID based on a mac address.
     * This is necessary because IDs have to be integers.
     *
     * @param  string $macAddress
     * @return string
     */
    private function _getId($macAddress)
    {
        $id = crc32($macAddress);
        return substr($id, 0, 8);
    }

    /**
     * Create new device.
     *
     * @param  array $data
     * @return array
     * @throws isys_exception_api
     */
    public function create_device($data)
    {
        $params = [];
        foreach (array_keys($this->_known_params) as $p) {
            if (! empty($data[$p])) {
                $params[$p] = $data[$p];
            }
        }
        return $this->_request($this->_proxy_address, 'POST', $params);
    }

    /**
     * Update existing device.
     *
     * @param  string $mac_address
     * @param  array $data
     * @return array
     * @throws isys_exception_api
     */
    public function update_device($mac_address, $data)
    {
        $uri = $this->_proxy_address . $mac_address;
        $params = [];
        foreach (array_keys($this->_known_params) as $p) {
            if (! empty($data[$p])) {
                $params[$p] = $data[$p];
            }
        }
        return $this->_request($uri, 'PUT', $params);
    }

    /**
     * Re-Index complete index.
     *
     * @return array
     * @throws isys_exception_api
     */
    public function re_index()
    {
        return $this->delete_device('_all_');
    }

    /**
     * Delete device.
     *
     * @param  string $mac_address
     * @return array
     * @throws isys_exception_api
     */
    public function delete_device($mac_address)
    {
        $uri = $this->_proxy_address . $mac_address;
        return $this->_request($uri, 'DELETE');
    }
}
