<?php

namespace idoit\Module\Feed;

use isys_application;
use isys_exception_api;
use idoit\Module\Feed\ProxyApi\VirtualDevice;

/**
 * Virtual Device Importer class.
 *
 * @package    i-doit
 * @subpackage smview_feed
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class VirtualDeviceImporter
{

    /**
     * List of required parameters.
     *
     * @var array
     */
    private $_requiredParameters = [
        'mac_address',
        'subnet',
        'device_name',
        'device',
        'port',
        'port_state',
        'last_seen',
    ];

    /**
     * Mapping between API and CSV parameters.
     *
     * @var array
     */
    private $_apiToCsvMapping = [
        'user_name'   => 'UserName',
        'mac_address' => 'MACAddress',
        'host_name'   => 'HostName',
        'ip_address'  => 'IPAddress',
        'subnet'      => 'Subnet',
        'device_name' => 'DeviceName',
        'device'      => 'Device',
        'port'        => 'Port',
        'port_name'   => 'PortName',
        'port_state'  => 'PortState',
        'vtp_domain'  => 'VTPDomain',
        'vlan'        => 'VLAN',
        'vlan_type'   => 'VLANType',
        'last_seen'   => 'LastSeen',
        'notes'       => 'Notes',
    ];

    /**
     * Logbook.
     *
     * @var array
     */
    private $_log = [];

    /**
     * Virtual device client.
     *
     * @var VirtualDevice
     */
    private $_virtual_device;

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->_virtual_device = new VirtualDevice();
    }

    /**
     * Import devices.
     *
     * @param  string $path
     * @return array
     */
    public function import($path)
    {
        $this->_log = [];
        $devices = $this->_getDevices($path);
        $posMapping = [];
        foreach ($devices as $rowIndex => $device) {
            if ($rowIndex == 0) {
                $posMapping = [];
                $indices = array_flip($device);
                foreach ($indices as $columnName => $index) {
                    $columnName = str_replace(' ', '', $columnName);
                    $columnName = strtolower($columnName);
                    $posMapping[$columnName] = $index;
                }
            } else {
                $this->_importDevice($device, $posMapping, $rowIndex);
            }
        }
        return $this->_log;
    }

    /**
     * Get devices from CSV file.
     *
     * @param  string $path
     * @return array
     */
    private function _getDevices($path)
    {
        $elements = [];
        if (($handle = fopen($path, "r")) !== FALSE) {
            while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
                $elements[] = $data;
            }
            fclose($handle);
        }
        return $elements;
    }

    /**
     * Import a device.
     *
     * @param  array $device
     * @param  array $posMapping
     * @param  int $rowIndex
     */
    private function _importDevice($device, $posMapping, $rowIndex)
    {
        $lang = isys_application::instance()->container->get('language');
        $dev = [];
        foreach ($this->_apiToCsvMapping as $api => $csv) {
            $csv = strtolower($csv);
            if (! isset($posMapping[$csv])) {
                // field not provided in CSV file
                continue;
            }
            $index = $posMapping[$csv];
            $dev[$api] = $device[$index];
        }

        $errors = [];
        foreach ($this->_requiredParameters as $param) {
            if (! isset($dev[$param]) || $dev[$param] == '') {
                $errors[] = $param;
            }
        }

        if (! empty($errors)) {
            $params = implode(', ', $errors);
            $msg = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__LOG_ERROR_MISSING_PARAMETERS');
            $this->_log[$rowIndex][] = "ERROR: $msg: " . $params;
            return;
        }

        if (empty($dev['host_name'])) {
            // use ip- or mac-address if host name is empty
            $dev['host_name'] = ! empty($dev['ip_address']) && $dev['ip_address'] != '0.0.0.0' ? $dev['ip_address'] : $dev['mac_address'];
        }

        $id = $dev['mac_address'];
        $dev['last_seen'] = str_replace('/', '-', $dev['last_seen']);
        try {
            $this->_virtual_device->create_device($dev);
            $msg = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__LOG_DEBUG_ADDED_NEW_DEVICE');
            $this->_log[$rowIndex][] = "DEBUG: $msg $id";
        } catch (isys_exception_api $e) {
            if ($e->getCode() == 409) {
                try {
                    $this->_virtual_device->update_device($id, $dev);
                    $msg = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__LOG_DEBUG_UPDATED_DEVICE');
                    $this->_log[$rowIndex][] = "DEBUG: $msg $id";
                } catch (isys_exception_api $ex) {
                    $msg = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__LOG_ERROR_UPDATED_DEVICE');
                    $this->_log[$rowIndex][] = "ERROR: $msg $id: " . $ex->getMessage();
                }
            } else {
                $msg = $lang->get('LC__CMDB__CATG__SMVIEW_FEED__LOG_ERROR_IMPORT_DEVICE');
                $this->_log[$rowIndex][] = "ERROR: $msg $id: " . $e->getMessage();
            }
        }
    }
}
